/****************************************************************************
** Meta object code from reading C++ file 'declarativecontentaction.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "declarativecontentaction.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativecontentaction.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DeclarativeContentAction_t {
    QByteArrayData data[15];
    char stringdata0[179];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DeclarativeContentAction_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DeclarativeContentAction_t qt_meta_stringdata_DeclarativeContentAction = {
    {
QT_MOC_LITERAL(0, 0, 24), // "DeclarativeContentAction"
QT_MOC_LITERAL(1, 25, 12), // "errorChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 15), // "mimeTypeChanged"
QT_MOC_LITERAL(4, 55, 7), // "trigger"
QT_MOC_LITERAL(5, 63, 3), // "url"
QT_MOC_LITERAL(6, 67, 5), // "error"
QT_MOC_LITERAL(7, 73, 5), // "Error"
QT_MOC_LITERAL(8, 79, 8), // "mimeType"
QT_MOC_LITERAL(9, 88, 7), // "NoError"
QT_MOC_LITERAL(10, 96, 20), // "FileTypeNotSupported"
QT_MOC_LITERAL(11, 117, 16), // "FileDoesNotExist"
QT_MOC_LITERAL(12, 134, 11), // "FileIsEmpty"
QT_MOC_LITERAL(13, 146, 21), // "UrlSchemeNotSupported"
QT_MOC_LITERAL(14, 168, 10) // "InvalidUrl"

    },
    "DeclarativeContentAction\0errorChanged\0"
    "\0mimeTypeChanged\0trigger\0url\0error\0"
    "Error\0mimeType\0NoError\0FileTypeNotSupported\0"
    "FileDoesNotExist\0FileIsEmpty\0"
    "UrlSchemeNotSupported\0InvalidUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DeclarativeContentAction[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       2,   34, // properties
       1,   42, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       4,    1,   31,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool, QMetaType::QUrl,    5,

 // properties: name, type, flags
       6, 0x80000000 | 7, 0x00495009,
       8, QMetaType::QString, 0x00495001,

 // properties: notify_signal_id
       0,
       1,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,    6,   47,

 // enum data: key, value
       9, uint(DeclarativeContentAction::NoError),
      10, uint(DeclarativeContentAction::FileTypeNotSupported),
      11, uint(DeclarativeContentAction::FileDoesNotExist),
      12, uint(DeclarativeContentAction::FileIsEmpty),
      13, uint(DeclarativeContentAction::UrlSchemeNotSupported),
      14, uint(DeclarativeContentAction::InvalidUrl),

       0        // eod
};

void DeclarativeContentAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeclarativeContentAction *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->errorChanged(); break;
        case 1: _t->mimeTypeChanged(); break;
        case 2: { bool _r = _t->trigger((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeclarativeContentAction::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeContentAction::errorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeclarativeContentAction::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeContentAction::mimeTypeChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DeclarativeContentAction *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Error*>(_v) = _t->error(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->mimeType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DeclarativeContentAction::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DeclarativeContentAction.data,
    qt_meta_data_DeclarativeContentAction,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DeclarativeContentAction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeContentAction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DeclarativeContentAction.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DeclarativeContentAction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DeclarativeContentAction::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DeclarativeContentAction::mimeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
